@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: MIT
  *
- * Copyright (C) 2018-2020 WireGuard LLC. All Rights Reserved.
+ * Copyright (C) 2018-2021 WireGuard LLC. All Rights Reserved.
  */
 
 package tun
@@ -8,6 +8,7 @@ package tun
 import (
 	"errors"
 	"fmt"
+	"log"
 	"os"
 	"sync/atomic"
 	"time"
@@ -43,15 +44,8 @@ type NativeTun struct {
 	readWait  windows.Handle
 }
 
-var WintunPool *wintun.Pool
-
-func init() {
-	var err error
-	WintunPool, err = wintun.MakePool("WireGuard")
-	if err != nil {
-		panic(fmt.Errorf("Failed to make pool: %w", err))
-	}
-}
+var WintunPool, _ = wintun.MakePool("WireGuard")
+var WintunStaticRequestedGUID *windows.GUID
 
 //go:linkname procyield runtime.procyield
 func procyield(cycles uint32)
@@ -64,7 +58,7 @@ func nanotime() int64
 // interface with the same name exist, it is reused.
 //
 func CreateTUN(ifname string, mtu int) (Device, error) {
-	return CreateTUNWithRequestedGUID(ifname, nil, mtu)
+	return CreateTUNWithRequestedGUID(ifname, WintunStaticRequestedGUID, mtu)
 }
 
 //
@@ -84,10 +78,13 @@ func CreateTUNWithRequestedGUID(ifname string, requestedGUID *windows.GUID, mtu
 			return nil, fmt.Errorf("Error deleting already existing interface: %w", err)
 		}
 	}
-	wt, _, err = WintunPool.CreateAdapter(ifname, requestedGUID)
+	wt, rebootRequired, err := WintunPool.CreateAdapter(ifname, requestedGUID)
 	if err != nil {
 		return nil, fmt.Errorf("Error creating interface: %w", err)
 	}
+	if rebootRequired {
+		log.Println("Windows indicated a reboot is required.")
+	}
 
 	forcedMTU := 1420
 	if mtu > 0 {
@@ -104,7 +101,7 @@ func CreateTUNWithRequestedGUID(ifname string, requestedGUID *windows.GUID, mtu
 
 	tun.session, err = wt.StartSession(0x800000) // Ring capacity, 8 MiB
 	if err != nil {
-		_, err = tun.wt.Delete(false)
+		tun.wt.Delete(false)
 		close(tun.events)
 		return nil, fmt.Errorf("Error starting session: %w", err)
 	}
